Attribute VB_Name = "FileSystemUtils"
Option Explicit

' ADOBE SYSTEMS INCORPORATED
' Copyright (C) 1994-2001 Adobe Systems Incorporated
' All rights reserved.
'
' NOTICE: Adobe permits you to use, modify, and distribute this file
' in accordance with the terms of the Adobe license agreement
' accompanying it. If you have received this file from a source other
' than Adobe, then your use, modification, or distribution of it
' requires the prior written permission of Adobe.

' Constants / Declarations
Public Const MAX_PATH = 260
Private Const BIF_RETURNONLYFSDIRS = 1
Private Const BIF_DONTGOBELOWDOMAIN = 2
Private Const OFN_FILEMUSTEXIST = &H1000

Dim gFileSystem As FileSystemObject

' Standard Open File Dialog
Private Declare Function GetOpenFileName Lib "comdlg32" Alias "GetOpenFileNameA" (pOpenfilename As OPENFILENAME) As Long

Private Type OPENFILENAME
    lStructSize As Long
    hwndOwner As Long
    hInstance As Long
    lpstrFilter As String
    lpstrCustomFilter As String
    nMaxCustFilter As Long
    nFilterIndex As Long
    lpstrFile As String
    nMaxFile As Long
    lpstrFileTitle As String
    nMaxFileTitle As Long
    lpstrInitialDir As String
    lpstrTitle As String
    flags As Long
    nFileOffset As Integer
    nFileExtension As Integer
    lpstrDefExt As String
    lCustData As Long
    lpfnHook As Long
    lpTemplateName As String
End Type

' FSQueryForFile
'
' Displays a standard open file dialog box.
' Returns the path selected by the user or
' the empty string if they cancelled.

Public Function FSQueryForFile(parent As Form) As String

    Dim OpenFile As OPENFILENAME
    Dim lReturn As Long
    Dim sFilter As String
    Dim sBuffer As String
    
    sFilter = "Catalog Index File (*.pdx)" & Chr(0) & "*.pdx" & Chr(0)
    
    With OpenFile
        .lStructSize = Len(OpenFile)
        .hwndOwner = parent.hWnd
        .hInstance = App.hInstance
        .lpstrFilter = sFilter
        .nFilterIndex = 1
        .lpstrFile = String(257, 0)
        .nMaxFile = Len(OpenFile.lpstrFile) - 1
        .lpstrFileTitle = OpenFile.lpstrFile
        .nMaxFileTitle = OpenFile.nMaxFile
        .lpstrInitialDir = "C:\"
        .lpstrTitle = "Select file to process..."
        .flags = OFN_FILEMUSTEXIST
    End With
    
    lReturn = GetOpenFileName(OpenFile)
    If lReturn = 0 Then
        sBuffer = ""
    Else
        sBuffer = Trim(OpenFile.lpstrFile)
    End If

    FSQueryForFile = sBuffer
End Function

' FSPopulateComboWithFilesFromFolder
'
' Populates a combobox with the names of
' the files in the specified folder.

Public Function FSPopulateComboWithFilesFromFolder(dlgItem As ComboBox, folder As String, extension As String)

    If (gFileSystem Is Nothing) Then
        Set gFileSystem = New FileSystemObject
    End If
    
    If (True = gFileSystem.FolderExists(folder)) Then
        
        Dim optionsFile As File
        
        For Each optionsFile In gFileSystem.GetFolder(folder).Files
          
            Dim strPos As Integer
            
            strPos = InStr(optionsFile.Name, extension)
            If Not (0 = strPos) Then
                dlgItem.AddItem Left$(optionsFile.Name, strPos - 1)
            End If
            
        Next
    End If ' FolderExists

End Function

